@@ -8,11 +8,20 @@
  *
  * Mon Jul  1 18:52:58 1996: janl@math.uio.no (Nicolai Langfeldt):
  *	Incorporated fix by Jonathan Kamens <jik@annex-1-slip-jik.cam.ov.com>
- * 1999-02-22 Arkadiusz Mikiewicz <misiek@pld.ORG.PL>
+ * 1999-02-22 Arkadiusz Miśkiewicz <misiek@pld.ORG.PL>
  * - added Native Language Support
  *	
  */
 
+/*
+ * This command is deprecated.  The utility is in maintenance mode,
+ * meaning we keep them in source tree for backward compatibility
+ * only.  Do not waste time making this command better, unless the
+ * fix is about security or other very critical issue.
+ *
+ * See Documentation/deprecated.txt for more information.
+ */
+
 #include <getopt.h>
 #include <limits.h>
 #include <stdio.h>
@@ -35,30 +44,32 @@
 
 static void __attribute__ ((__noreturn__)) usage(FILE * out)
 {
-	fprintf(out,
-		_("Usage: %s [options] [-t type fs-options] device [size]\n"),
-		program_invocation_short_name);
+	fputs(USAGE_HEADER, out);
+	fprintf(out, _(" %s [options] [-t <type>] [fs-options] <device> [<size>]\n"),
+		     program_invocation_short_name);
 
-	fprintf(out, _("\nOptions:\n"
-		       " -t, --type=TYPE  file system type, when undefined ext2 is used\n"
-		       "     fs-options   parameters to real file system builder\n"
-		       "     device       path to a device\n"
-		       "     size         number of blocks on the device\n"
-		       " -V, --verbose    explain what is done\n"
-		       "                  defining -V more than once will cause a dry-run\n"
-		       " -V, --version    output version information and exit\n"
-		       "                  -V as version must be only option\n"
-		       " -h, --help       display this help and exit\n"));
+	fputs(USAGE_SEPARATOR, out);
+	fputs(_("Make a Linux filesystem.\n"), out);
 
-	fprintf(out, _("\nFor more information see mkfs(8).\n"));
+	fputs(USAGE_OPTIONS, out);
+	fprintf(out, _(" -t, --type=<type>  filesystem type; when unspecified, ext2 is used\n"));
+	fprintf(out, _("     fs-options     parameters for the real filesystem builder\n"));
+	fprintf(out, _("     <device>       path to the device to be used\n"));
+	fprintf(out, _("     <size>         number of blocks to be used on the device\n"));
+	fprintf(out, _(" -V, --verbose      explain what is being done;\n"
+		       "                      specifying -V more than once will cause a dry-run\n"));
+	fprintf(out, _(" -V, --version      display version information and exit;\n"
+		       "                      -V as --version must be the only option\n"));
+	fprintf(out, _(" -h, --help         display this help text and exit\n"));
+
+	fprintf(out, USAGE_MAN_TAIL("mkfs(8)"));
 
 	exit(out == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
 }
 
 static void __attribute__ ((__noreturn__)) print_version(void)
 {
-	printf(_("%s (%s)\n"),
-	       program_invocation_short_name, PACKAGE_STRING);
+	printf(UTIL_LINUX_VERSION);
 	exit(EXIT_SUCCESS);
 }
 
@@ -74,6 +85,7 @@ int main(int argc, char **argv)
 	static const struct option longopts[] = {
 		{"type", required_argument, NULL, 't'},
 		{"version", no_argument, NULL, VERSION_OPTION},
+		{"verbose", no_argument, NULL, 'V'},
 		{"help", no_argument, NULL, 'h'},
 		{NULL, 0, NULL, 0}
 	};
@@ -128,7 +140,7 @@ int main(int argc, char **argv)
 	argv[--optind] = progname;
 
 	if (verbose) {
-		printf(_("mkfs (%s)\n"), PACKAGE_STRING);
+		printf(UTIL_LINUX_VERSION);
 		i = optind;
 		while (argv[i])
 			printf("%s ", argv[i++]);
@@ -139,6 +151,5 @@ int main(int argc, char **argv)
 
 	/* Execute the program */
 	execvp(progname, argv + optind);
-	perror(progname);
-	return EXIT_FAILURE;
+	err(EXIT_FAILURE, _("failed to execute %s"), progname);
 }
