@@ -31,7 +31,7 @@ extern "C" {
 #endif
 
 #define BLKID_VERSION   "2.26.0"
-#define BLKID_DATE      "19-Feb-2015"
+#define BLKID_DATE      "13-Mar-2015"
 
 /**
  * blkid_dev:
