@@ -8,10 +8,10 @@
 /**
  * SECTION: optstr
  * @title: Options string
- * @short_description: low-level API for work with mount options
+ * @short_description: low-level API for working with mount options
  *
- * This is simple and low-level API to work with mount options that are stored
- * in string.
+ * This is a simple and low-level API to working with mount options that are stored
+ * in a string.
  */
 #include <ctype.h>
 
@@ -35,9 +35,12 @@ struct libmnt_optloc {
 
 #define mnt_init_optloc(_ol)	(memset((_ol), 0, sizeof(struct libmnt_optloc)))
 
+#define mnt_optmap_entry_novalue(e) \
+		(e && (e)->name && !strchr((e)->name, '=') && !((e)->mask & MNT_PREFIX))
+
 /*
- * Parses the first option from @optstr. The @optstr pointer is set to begin of
- * the next option.
+ * Parses the first option from @optstr. The @optstr pointer is set to the beginning
+ * of the next option.
  *
  * Returns -EINVAL on parse error, 1 at the end of optstr and 0 on success.
  */
@@ -69,12 +72,12 @@ static int mnt_optstr_parse_next(char **optstr,	 char **name, size_t *namesz,
 
 	for (p = optstr0; p && *p; p++) {
 		if (!start)
-			start = p;		/* begin of the option item */
+			start = p;		/* beginning of the option item */
 		if (*p == '"')
 			open_quote ^= 1;	/* reverse the status */
 		if (open_quote)
 			continue;		/* still in quoted block */
-		if (!sep && *p == '=')
+		if (!sep && p > start && *p == '=')
 			sep = p;		/* name and value separator */
 		if (*p == ',')
 			stop = p;		/* terminate the option item */
@@ -103,12 +106,12 @@ static int mnt_optstr_parse_next(char **optstr,	 char **name, size_t *namesz,
 	return 1;				/* end of optstr */
 
 error:
-	DBG(OPTIONS, mnt_debug("parse error: \"%s\"", optstr0));
+	DBG(OPTIONS, ul_debug("parse error: \"%s\"", optstr0));
 	return -EINVAL;
 }
 
 /*
- * Locates the first option that match with @name. The @end is set to
+ * Locates the first option that matches @name. The @end is set to the
  * char behind the option (it means ',' or \0).
  *
  * Returns negative number on parse error, 1 when not found and 0 on success.
@@ -124,7 +127,6 @@ static int mnt_optstr_locate_option(char *optstr, const char *name,
 		return 1;
 
 	assert(name);
-	assert(optstr);
 
 	namesz = strlen(name);
 
@@ -147,11 +149,11 @@ static int mnt_optstr_locate_option(char *optstr, const char *name,
 
 /**
  * mnt_optstr_next_option:
- * @optstr: option string, returns position to next option
- * @name: returns option name
- * @namesz: returns option name length
- * @value: returns option value or NULL
- * @valuesz: returns option value length or zero
+ * @optstr: option string, returns the position of the next option
+ * @name: returns the option name
+ * @namesz: returns the option name length
+ * @value: returns the option value or NULL
+ * @valuesz: returns the option value length or zero
  *
  * Parses the first option in @optstr.
  *
@@ -176,6 +178,7 @@ static int __mnt_optstr_append_option(char **optstr,
 	assert(name);
 	assert(*name);
 	assert(nsz);
+	assert(optstr);
 
 	osz = *optstr ? strlen(*optstr) : 0;
 
@@ -210,17 +213,19 @@ static int __mnt_optstr_append_option(char **optstr,
 
 /**
  * mnt_optstr_append_option:
- * @optstr: option string or NULL, returns reallocated string
+ * @optstr: option string or NULL, returns a reallocated string
  * @name: value name
  * @value: value
  *
- * Returns: 0 on success or -1 in case of error. After error the @optstr should
+ * Returns: 0 on success or <0 in case of error. After an error the @optstr should
  *          be unmodified.
  */
 int mnt_optstr_append_option(char **optstr, const char *name, const char *value)
 {
 	size_t vsz, nsz;
 
+	if (!optstr)
+		return -EINVAL;
 	if (!name || !*name)
 		return 0;
 
@@ -232,11 +237,11 @@ int mnt_optstr_append_option(char **optstr, const char *name, const char *value)
 
 /**
  * mnt_optstr_prepend_option:
- * @optstr: option string or NULL, returns reallocated string
+ * @optstr: option string or NULL, returns a reallocated string
  * @name: value name
  * @value: value
  *
- * Returns: 0 on success or -1 in case of error. After error the @optstr should
+ * Returns: 0 on success or <0 in case of error. After an error the @optstr should
  *          be unmodified.
  */
 int mnt_optstr_prepend_option(char **optstr, const char *name, const char *value)
@@ -244,6 +249,11 @@ int mnt_optstr_prepend_option(char **optstr, const char *name, const char *value
 	int rc = 0;
 	char *tmp = *optstr;
 
+	if (!optstr)
+		return -EINVAL;
+	if (!name || !*name)
+		return 0;
+
 	*optstr = NULL;
 
 	rc = mnt_optstr_append_option(optstr, name, value);
@@ -257,16 +267,16 @@ int mnt_optstr_prepend_option(char **optstr, const char *name, const char *value
 	free(*optstr);
 	*optstr = tmp;
 
-	DBG(OPTIONS, mnt_debug("failed to prepend '%s[=%s]' to '%s'",
+	DBG(OPTIONS, ul_debug("failed to prepend '%s[=%s]' to '%s'",
 				name, value, *optstr));
 	return rc;
 }
 
 /**
  * mnt_optstr_get_option:
- * @optstr: string with comma separated list of options
+ * @optstr: string with a comma separated list of options
  * @name: requested option name
- * @value: returns pointer to the begin of the value (e.g. name=VALUE) or NULL
+ * @value: returns a pointer to the beginning of the value (e.g. name=VALUE) or NULL
  * @valsz: returns size of the value or 0
  *
  * Returns: 0 on success, 1 when not found the @name or negative number in case
@@ -278,6 +288,9 @@ int mnt_optstr_get_option(const char *optstr, const char *name,
 	struct libmnt_optloc ol;
 	int rc;
 
+	if (!optstr || !name)
+		return -EINVAL;
+
 	mnt_init_optloc(&ol);
 
 	rc = mnt_optstr_locate_option((char *) optstr, name, &ol);
@@ -292,7 +305,7 @@ int mnt_optstr_get_option(const char *optstr, const char *name,
 
 /**
  * mnt_optstr_deduplicate_option:
- * @optstr: string with comma separated list of options
+ * @optstr: string with a comma separated list of options
  * @name: requested option name
  *
  * Removes all instances of @name except the last one.
@@ -303,8 +316,12 @@ int mnt_optstr_get_option(const char *optstr, const char *name,
 int mnt_optstr_deduplicate_option(char **optstr, const char *name)
 {
 	int rc;
-	char *begin = NULL, *end = NULL, *opt = *optstr;
+	char *begin = NULL, *end = NULL, *opt;
 
+	if (!optstr || !name)
+		return -EINVAL;
+
+	opt = *optstr;
 	do {
 		struct libmnt_optloc ol;
 
@@ -313,12 +330,12 @@ int mnt_optstr_deduplicate_option(char **optstr, const char *name)
 		rc = mnt_optstr_locate_option(opt, name, &ol);
 		if (!rc) {
 			if (begin) {
-				/* remove previous instance */
+				/* remove the previous instance */
 				size_t shift = strlen(*optstr);
 
 				mnt_optstr_remove_option_at(optstr, begin, end);
 
-				/* now all offset are not valied anymore - recount */
+				/* now all the offsets are not valid anymore - recount */
 				shift -= strlen(*optstr);
 				ol.begin -= shift;
 				ol.end -= shift;
@@ -333,7 +350,7 @@ int mnt_optstr_deduplicate_option(char **optstr, const char *name)
 }
 
 /*
- * The result never starts or ends with comma or contains two commas
+ * The result never starts or ends with a comma or contains two commas
  *    (e.g. ",aaa,bbb" or "aaa,,bbb" or "aaa,")
  */
 int mnt_optstr_remove_option_at(char **optstr, char *begin, char *end)
@@ -356,7 +373,8 @@ int mnt_optstr_remove_option_at(char **optstr, char *begin, char *end)
 }
 
 /* insert 'substr' or '=substr' to @str on position @pos */
-static int insert_value(char **str, char *pos, const char *substr, char **next)
+static int __attribute__((nonnull(1,2,3)))
+insert_value(char **str, char *pos, const char *substr, char **next)
 {
 	size_t subsz = strlen(substr);			/* substring size */
 	size_t strsz = strlen(*str);
@@ -368,14 +386,14 @@ static int insert_value(char **str, char *pos, const char *substr, char **next)
 	/* is it necessary to prepend '=' before the substring ? */
 	sep = !(pos > *str && *(pos - 1) == '=');
 
-	/* save an offset of the place where we need add substr */
+	/* save an offset of the place where we need to add substr */
 	posoff = pos - *str;
 
 	p = realloc(*str, strsz + sep + subsz + 1);
 	if (!p)
 		return -ENOMEM;
 
-	/* zeroize new allocated memory -- valgind loves is... */
+	/* zeroize the newly allocated memory -- valgrind loves us... */
 	memset(p + strsz, 0, sep + subsz + 1);
 
 	/* set pointers to the reallocated string */
@@ -383,7 +401,7 @@ static int insert_value(char **str, char *pos, const char *substr, char **next)
 	pos = p + posoff;
 
 	if (possz)
-		/* create a room for new substring */
+		/* create a room for the new substring */
 		memmove(pos + subsz + sep, pos, possz + 1);
 	if (sep)
 		*pos++ = '=';
@@ -401,11 +419,11 @@ static int insert_value(char **str, char *pos, const char *substr, char **next)
 
 /**
  * mnt_optstr_set_option:
- * @optstr: string with comma separated list of options
+ * @optstr: string with a comma separated list of options
  * @name: requested option
  * @value: new value or NULL
  *
- * Set or unset option @value.
+ * Set or unset the option @value.
  *
  * Returns: 0 on success, 1 when not found the @name or negative number in case
  * of error.
@@ -416,7 +434,7 @@ int mnt_optstr_set_option(char **optstr, const char *name, const char *value)
 	char *nameend;
 	int rc = 1;
 
-	if (!optstr)
+	if (!optstr || !name)
 		return -EINVAL;
 
 	mnt_init_optloc(&ol);
@@ -451,7 +469,7 @@ int mnt_optstr_set_option(char **optstr, const char *name, const char *value)
 
 /**
  * mnt_optstr_remove_option:
- * @optstr: string with comma separated list of options
+ * @optstr: string with a comma separated list of options
  * @name: requested option name
  *
  * Returns: 0 on success, 1 when not found the @name or negative number in case
@@ -462,6 +480,9 @@ int mnt_optstr_remove_option(char **optstr, const char *name)
 	struct libmnt_optloc ol;
 	int rc;
 
+	if (!optstr || !name)
+		return -EINVAL;
+
 	mnt_init_optloc(&ol);
 
 	rc = mnt_optstr_locate_option(*optstr, name, &ol);
@@ -485,13 +506,13 @@ int mnt_optstr_remove_option(char **optstr, const char *name)
  *
  *	mnt_split_optstr(optstr, &u, NULL, NULL, MNT_NOMTAB, 0);
  *
- * returns all userspace options, the options that does not belong to
+ * returns all userspace options, the options that do not belong to
  * mtab are ignored.
  *
  * Note that FS options are all options that are undefined in MNT_USERSPACE_MAP
  * or MNT_LINUX_MAP.
  *
- * Returns: 0 on success, or negative number in case of error.
+ * Returns: 0 on success, or a negative number in case of error.
  */
 int mnt_split_optstr(const char *optstr, char **user, char **vfs,
 		     char **fs, int ignore_user, int ignore_vfs)
@@ -500,8 +521,6 @@ int mnt_split_optstr(const char *optstr, char **user, char **vfs,
 	size_t namesz, valsz;
 	struct libmnt_optmap const *maps[2];
 
-	assert(optstr);
-
 	if (!optstr)
 		return -EINVAL;
 
@@ -515,7 +534,7 @@ int mnt_split_optstr(const char *optstr, char **user, char **vfs,
 	if (user)
 		*user = NULL;
 
-	while(!mnt_optstr_next_option(&str, &name, &namesz, &val, &valsz)) {
+	while (!mnt_optstr_next_option(&str, &name, &namesz, &val, &valsz)) {
 		int rc = 0;
 		const struct libmnt_optmap *ent = NULL;
 		const struct libmnt_optmap *m =
@@ -524,6 +543,10 @@ int mnt_split_optstr(const char *optstr, char **user, char **vfs,
 		if (ent && !ent->id)
 			continue;	/* ignore undefined options (comments) */
 
+		/* ignore name=<value> if options map expects <name> only */
+		if (valsz && mnt_optmap_entry_novalue(ent))
+			m = NULL;
+
 		if (ent && m && m == maps[0] && vfs) {
 			if (ignore_vfs && (ent->mask & ignore_vfs))
 				continue;
@@ -538,12 +561,18 @@ int mnt_split_optstr(const char *optstr, char **user, char **vfs,
 			rc = __mnt_optstr_append_option(fs, name, namesz,
 								val, valsz);
 		if (rc) {
-			if (vfs)
+			if (vfs) {
 				free(*vfs);
-			if (fs)
+				*vfs = NULL;
+			}
+			if (fs) {
 				free(*fs);
-			if (user)
+				*fs = NULL;
+			}
+			if (user) {
 				free(*user);
+				*user = NULL;
+			}
 			return rc;
 		}
 	}
@@ -553,21 +582,21 @@ int mnt_split_optstr(const char *optstr, char **user, char **vfs,
 
 /**
  * mnt_optstr_get_options
- * @optstr: string with comma separated list of options
+ * @optstr: string with a comma separated list of options
  * @subset: returns newly allocated string with options
  * @map: options map
  * @ignore: mask of the options that should be ignored
  *
- * Extracts options from @optstr that belongs to the @map, for example:
+ * Extracts options from @optstr that belong to the @map, for example:
  *
  *	 mnt_optstr_get_options(optstr, &p,
  *			mnt_get_builtin_optmap(MNT_LINUX_MAP),
  *			MNT_NOMTAB);
  *
- * the 'p' returns all VFS options, the options that does not belong to mtab
+ * the 'p' returns all VFS options, the options that do not belong to mtab
  * are ignored.
  *
- * Returns: 0 on success, or negative number in case of error.
+ * Returns: 0 on success, or a negative number in case of error.
  */
 int mnt_optstr_get_options(const char *optstr, char **subset,
 			    const struct libmnt_optmap *map, int ignore)
@@ -593,6 +622,11 @@ int mnt_optstr_get_options(const char *optstr, char **subset,
 
 		if (ignore && (ent->mask & ignore))
 			continue;
+
+		/* ignore name=<value> if options map expects <name> only */
+		if (valsz && mnt_optmap_entry_novalue(ent))
+			continue;
+
 		rc = __mnt_optstr_append_option(subset, name, namesz, val, valsz);
 		if (rc) {
 			free(*subset);
@@ -618,8 +652,8 @@ int mnt_optstr_get_options(const char *optstr, char **subset,
  *
  *	"bind,noexec,foo,bar" --returns->   MS_BIND|MS_NOEXEC
  *
- * Note that @flags are not zeroized by this function! This function set/unset
- * bites in the @flags only.
+ * Note that @flags are not zeroized by this function! This function sets/unsets
+ * bits in the @flags only.
  *
  * Returns: 0 on success or negative number in case of error
  */
@@ -631,8 +665,6 @@ int mnt_optstr_get_flags(const char *optstr, unsigned long *flags,
 	size_t namesz = 0, valsz = 0;
 	int nmaps = 0;
 
-	assert(optstr);
-
 	if (!optstr || !flags || !map)
 		return -EINVAL;
 
@@ -653,6 +685,10 @@ int mnt_optstr_get_flags(const char *optstr, unsigned long *flags,
 		if (!m || !ent || !ent->id)
 			continue;
 
+		/* ignore name=<value> if options map expects <name> only */
+		if (valsz && mnt_optmap_entry_novalue(ent))
+			continue;
+
 		if (m == map) {				/* requested map */
 			if (ent->mask & MNT_INVERT)
 				*flags &= ~ent->id;
@@ -697,20 +733,18 @@ int mnt_optstr_apply_flags(char **optstr, unsigned long flags,
 	unsigned long fl;
 	int rc = 0;
 
-	assert(optstr);
-
 	if (!optstr || !map)
 		return -EINVAL;
 
-	DBG(CXT, mnt_debug("applying 0x%08lu flags to '%s'", flags, *optstr));
+	DBG(CXT, ul_debug("applying 0x%08lu flags to '%s'", flags, *optstr));
 
 	maps[0] = map;
 	next = *optstr;
 	fl = flags;
 
 	/*
-	 * There is a convetion that 'rw/ro' flags is always at the begin of
-	 * the string (athough the 'rw' is unnecessary).
+	 * There is a convention that 'rw/ro' flags are always at the beginning of
+	 * the string (although the 'rw' is unnecessary).
 	 */
 	if (map == mnt_get_builtin_optmap(MNT_LINUX_MAP)) {
 		const char *o = (fl & MS_RDONLY) ? "ro" : "rw";
@@ -736,7 +770,7 @@ int mnt_optstr_apply_flags(char **optstr, unsigned long flags,
 	if (next && *next) {
 		/*
 		 * scan @optstr and remove options that are missing in
-		 * the @flags
+		 * @flags
 		 */
 		while(!mnt_optstr_next_option(&next, &name, &namesz,
 							&val, &valsz)) {
@@ -746,8 +780,12 @@ int mnt_optstr_apply_flags(char **optstr, unsigned long flags,
 				/*
 				 * remove unwanted option (rw/ro is already set)
 				 */
-				if (!ent->id)
+				if (!ent || !ent->id)
 					continue;
+				/* ignore name=<value> if options map expects <name> only */
+				if (valsz && mnt_optmap_entry_novalue(ent))
+					continue;
+
 				if (ent->id == MS_RDONLY ||
 				    (ent->mask & MNT_INVERT) ||
 				    (fl & ent->id) != (unsigned long) ent->id) {
@@ -797,10 +835,10 @@ int mnt_optstr_apply_flags(char **optstr, unsigned long flags,
 		}
 	}
 
-	DBG(CXT, mnt_debug("new optstr '%s'", *optstr));
+	DBG(CXT, ul_debug("new optstr '%s'", *optstr));
 	return rc;
 err:
-	DBG(CXT, mnt_debug("failed to apply flags [rc=%d]", rc));
+	DBG(CXT, ul_debug("failed to apply flags [rc=%d]", rc));
 	return rc;
 }
 
@@ -814,7 +852,7 @@ err:
  * modify @optstr and returns zero if libmount is compiled without SELinux
  * support.
  *
- * Returns: 0 on success, negative number in case of error.
+ * Returns: 0 on success, a negative number in case of error.
  */
 #ifndef HAVE_LIBSELINUX
 int mnt_optstr_fix_secontext(char **optstr __attribute__ ((__unused__)),
@@ -839,7 +877,7 @@ int mnt_optstr_fix_secontext(char **optstr,
 	if (!optstr || !*optstr || !value || !valsz)
 		return -EINVAL;
 
-	DBG(CXT, mnt_debug("fixing SELinux context"));
+	DBG(CXT, ul_debug("fixing SELinux context"));
 
 	begin = value;
 	end = value + valsz;
@@ -860,7 +898,7 @@ int mnt_optstr_fix_secontext(char **optstr,
 	/* translate the context */
 	rc = selinux_trans_to_raw_context((security_context_t) p, &raw);
 
-	DBG(CXT, mnt_debug("SELinux context '%s' translated to '%s'",
+	DBG(CXT, ul_debug("SELinux context '%s' translated to '%s'",
 			p, rc == -1 ? "FAILED" : (char *) raw));
 
 	free(p);
@@ -868,7 +906,7 @@ int mnt_optstr_fix_secontext(char **optstr,
 		return -EINVAL;
 
 
-	/* create quoted string from the raw context */
+	/* create a quoted string from the raw context */
 	sz = strlen((char *) raw);
 	if (!sz)
 		return -EINVAL;
@@ -905,8 +943,8 @@ static int set_uint_value(char **optstr, unsigned int num,
 }
 
 /*
- * @optstr: string with comma separated list of options
- * @value: pointer to the begin of the uid value
+ * @optstr: string with a comma separated list of options
+ * @value: pointer to the beginning of the uid value
  * @valsz: size of the value
  * @next: returns pointer to the next option (optional argument)
 
@@ -916,7 +954,7 @@ static int set_uint_value(char **optstr, unsigned int num,
  *	if (!mnt_optstr_get_option(optstr, "uid", &val, &valsz))
  *		mnt_optstr_fix_uid(&optstr, val, valsz, NULL);
  *
- * Returns: 0 on success, negative number in case of error.
+ * Returns: 0 on success, a negative number in case of error.
  */
 int mnt_optstr_fix_uid(char **optstr, char *value, size_t valsz, char **next)
 {
@@ -926,7 +964,7 @@ int mnt_optstr_fix_uid(char **optstr, char *value, size_t valsz, char **next)
 	if (!optstr || !*optstr || !value || !valsz)
 		return -EINVAL;
 
-	DBG(CXT, mnt_debug("fixing uid"));
+	DBG(CXT, ul_debug("fixing uid"));
 
 	end = value + valsz;
 
@@ -956,14 +994,14 @@ int mnt_optstr_fix_uid(char **optstr, char *value, size_t valsz, char **next)
 }
 
 /*
- * @optstr: string with comma separated list of options
- * @value: pointer to the begin of the uid value
+ * @optstr: string with a comma separated list of options
+ * @value: pointer to the beginning of the uid value
  * @valsz: size of the value
  * @next: returns pointer to the next option (optional argument)
 
  * Translates "groupname" or "usergid" to the real GID.
  *
- * Returns: 0 on success, negative number in case of error.
+ * Returns: 0 on success, a negative number in case of error.
  */
 int mnt_optstr_fix_gid(char **optstr, char *value, size_t valsz, char **next)
 {
@@ -973,7 +1011,7 @@ int mnt_optstr_fix_gid(char **optstr, char *value, size_t valsz, char **next)
 	if (!optstr || !*optstr || !value || !valsz)
 		return -EINVAL;
 
-	DBG(CXT, mnt_debug("fixing gid"));
+	DBG(CXT, ul_debug("fixing gid"));
 
 	end = value + valsz;
 
@@ -1012,7 +1050,7 @@ int mnt_optstr_fix_user(char **optstr)
 	struct libmnt_optloc ol;
 	int rc = 0;
 
-	DBG(CXT, mnt_debug("fixing user"));
+	DBG(CXT, ul_debug("fixing user"));
 
 	mnt_init_optloc(&ol);
 
@@ -1286,7 +1324,7 @@ int main(int argc, char *argv[])
 {
 	struct libmnt_test tss[] = {
 		{ "--append", test_append, "<optstr> <name> [<value>]  append value to optstr" },
-		{ "--prepend",test_prepend,"<optstr> <name> [<value>]  prepend  value to optstr" },
+		{ "--prepend",test_prepend,"<optstr> <name> [<value>]  prepend value to optstr" },
 		{ "--set",    test_set,    "<optstr> <name> [<value>]  (un)set value" },
 		{ "--get",    test_get,    "<optstr> <name>            search name in optstr" },
 		{ "--remove", test_remove, "<optstr> <name>            remove name in optstr" },
